//--------------------------------------------------------------------
// UTCN, Catedra de Calculatoare
// Sisteme de intrare/iesire si echipamente periferice
//--------------------------------------------------------------------
// http://users.utcluj.ro/~baruch/ro/pages/cursuri/sisteme-de-intrareiesire/laborator.php
//--------------------------------------------------------------------
// Fisier:		Menu.cpp
// Data:		10.08.2005
// Modificat:	20.02.2009
//--------------------------------------------------------------------
// Fisier sablon pentru fereastra unei aplicatii Windows
//--------------------------------------------------------------------
// - Meniuri:	File -> Open
//				File -> Close
//				File -> Exit
//				Operations -> Test porturi paralele
//				Operations -> Operation 1 (fara parametri)
//				Operations -> Operation 2 (fara parametri)
//				Operations -> Operation 3 (fara parametri)
//				Help -> About
//--------------------------------------------------------------------
// - Functii Win32 API utilizate:
//   CloseHandle, CreateFile, CreateFont, CreateWindowEx, DefWindowProc, 
//   DestroyWindow, DialogBox, DispatchMessage, EndDialog, GetDC, 
//   GetDeviceCaps, GetDlgItem, GetFileSize, GetMessage, GetModuleHandle,
//   GetOpenFileName, GetStockObject, GetSysColor, GetSystemMetrics, 
//   GlobalAlloc, GlobalFree, LoadImage, MessageBox, MulDiv, 
//   PostQuitMessage, PostMessage, ReadFile, RegisterClassEx, ReleaseDC,
//   SelectObject, SendMessage, SetBkColor, SetWindowText, ShowWindow, 
//   TranslateMessage, UpdateWindow, WndProc, ZeroMemory 
//--------------------------------------------------------------------
// - Sistem de operare: Microsoft Windows 2000/XP
// - Compilator: Microsoft Visual Studio 2008
//--------------------------------------------------------------------

#include <windows.h>
#include "resource.h"

#define WIN32_LEAN_AND_MEAN		// optional: fara MFC
//#define UNICODE				// optional: pentru Windows 2000/XP

//--------------------------------------------------------------------
// De completat: Declaratiile functiilor externe
void TestPort (HWND hWnd, HDC hDc);
//--------------------------------------------------------------------

// Declaratiile functiilor utilizate
LRESULT CALLBACK WndProc (HWND hWnd, UINT uMes,
						  WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutDlgProc (HWND hWnd, UINT uMes,
						  WPARAM wParam, LPARAM lParam);
int ExecFileOpen (HWND hWnd);
HDC InitTextOut (HWND hWnd);

//--------------------------------------------------------------------
// Functia ExecFileOpen
//--------------------------------------------------------------------
// 
// Functie:		- Executa dialogul pentru selectarea fisierului care
//				  va fi deschis
//				- Deschide fisierul
//				- Citeste fisierul intr-un buffer
//				- Afiseaza continutul fisierului intr-o fereastra
//				  de editare
//
// Parametri:	hWnd - Indicator la fereastra aplicatiei
//
// Returneaza:	0 - Fisierul a fost deschis si citit cu succes
//				1 - Dialogul pentru deschidere a fost abandonat
//				2 - Eroare la deschiderea fisierului
//				3 - Eroare la alocarea memoriei
//				4 - Eroare la citirea fisierului
//
//--------------------------------------------------------------------

int ExecFileOpen (HWND hWnd)
{
	HANDLE hFile;				// indicator la fisier
	OPENFILENAME ofn;			// structura pentru apelul functiei 
								// de dialog GetOpenFileName
	char szFileName [MAX_PATH+1] = "";
								// buffer pentru calea si numele fisierului
	DWORD dwFileSize;			// dimensiunea fisierului
	LPSTR pszBuffer;			// pointer la bufferul pentru fisier
	DWORD dwRead;				// numarul de octeti cititi
	HWND hDisplay;				// indicator la fereastra pentru afisare
	HGDIOBJ hfFont;

	ZeroMemory (&ofn, sizeof (ofn));
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hWnd;
	ofn.lpstrFilter = "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
	ofn.lpstrFile = szFileName;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrInitialDir = "D:\\Laborator\\SIE\\";
	ofn.Flags = OFN_HIDEREADONLY;
	ofn.lpstrDefExt = "txt";

	if (GetOpenFileName (&ofn) == 0)
		return 1;				// dialogul pentru selectia fisierului
								// a fost abandonat
	hFile = CreateFile (szFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_ALWAYS, 0, NULL);
	if (hFile == INVALID_HANDLE_VALUE) {
		MessageBox (hWnd, "Eroare la deschiderea fisierului",
					"Aplicatie SIE", MB_OK | MB_ICONERROR);
		return 2;				// eroare la deschiderea fisierului
	}
	dwFileSize = GetFileSize (hFile, NULL);
	pszBuffer = (LPSTR) GlobalAlloc (GPTR, dwFileSize + 1);
								// aloca memorie pentru buffer
	if (pszBuffer == NULL)
		return 3;				// eroare la alocarea memoriei
	if (!ReadFile (hFile, pszBuffer, dwFileSize, &dwRead, NULL)) {
		MessageBox (hWnd, "Eroare la citirea fisierului",
					"Aplicatie SIE", MB_OK | MB_ICONERROR);
		return 4;				// eroare la citirea fisierului
	}
	pszBuffer [dwFileSize] = 0;	// adauga un octet 0 la sfarsit

	hDisplay = CreateWindowEx (	// creeaza fereastra pentru afisare
		WS_EX_CLIENTEDGE,
		"EDIT",					// numele clasei sistem Edit
		"FisierText",			// numele ferestrei
		WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL |
		ES_MULTILINE | ES_AUTOVSCROLL,
		0,						// pozitia orizontala
		0,						// pozitia verticala
		GetSystemMetrics (SM_CXSCREEN)/2 - 10,
		GetSystemMetrics (SM_CYSCREEN)/2 - 50,
		hWnd,					// indicator la fereastra parinte
		(HMENU) IDC_DISPLAY,	// identificatorul ferestrei de afisare
		GetModuleHandle (NULL),	// indicator la instanta modulului
		NULL);
	if (hDisplay == NULL)
		MessageBox (hWnd, "Eroare la crearea ferestrei pentru afisare",
					"Eroare", MB_OK | MB_ICONERROR);
	hfFont = GetStockObject (DEFAULT_GUI_FONT);
	SendMessage (hDisplay, WM_SETFONT, (WPARAM) hfFont,
		(LPARAM) FALSE);		// seteaza font-ul
	SetWindowText (hDisplay, pszBuffer);
								// afiseaza textul
	GlobalFree (pszBuffer);		// elibereaza memoria
	CloseHandle (hFile);		// inchide fisierul
	return 0;
}

//--------------------------------------------------------------------
// Functia InitTextOut
//--------------------------------------------------------------------
//
// Functie:		- Determina contextul dispozitivului de afisare
//				- Seteaza tipul caracterelor la tipul implicit utilizat
//				  pentru meniuri si cutii de dialog (MS Sans Serif)
//				- Seteaza marimea caracterelor la 9 puncte
//				- Seteaza culoarea de fond la COLOR_BTNFACE
//
// Parametri:	hWnd - Indicator la fereastra aplicatiei
//
// Returneaza:	Indicatorul la contextul dispozitivului de afisare
//
//--------------------------------------------------------------------

HDC InitTextOut (HWND hWnd)
{
	HDC hDc;					// indicator la contextul dispozitivului
	HGDIOBJ hfFont;
	int lfSize;
	DWORD syscol;

	hDc = GetDC (hWnd);			// determina contextul dispozitivului
	lfSize = -MulDiv (9, GetDeviceCaps (hDc, LOGPIXELSY), 72);
								// determina dimensiunea in unitati logice
	hfFont = CreateFont (lfSize, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						 "MS Sans Serif");	// creeaza un font logic
	if (hfFont == NULL)
		hfFont = GetStockObject (DEFAULT_GUI_FONT);
	SelectObject (hDc, hfFont);	// seteaza noul font
	syscol = GetSysColor (COLOR_BTNFACE);
								// determina valoarea RGB a culorii sistem
	SetBkColor (hDc, syscol);	// seteaza culoarea de fond
//	SetBkMode (hDc, TRANSPARENT);
	return hDc;
}

//--------------------------------------------------------------------
// Functia WinMain
//--------------------------------------------------------------------

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
					LPSTR lpCmdLine, int nCmdShow)
{
	WNDCLASSEX Wnd;				// structura pentru clasa de ferestre
	HWND hWnd;					// indicator la fereastra
	MSG Msg;					// structura pentru mesaje
	HICON hIcon;
	HICON hIconSm;
	HCURSOR hCursor;
	const char szClassName [] = "SieWindowClass";

// Initializeaza iconitele si cursorul ferestrei
	hIcon = (HICON) LoadImage (NULL, "sie.ico", IMAGE_ICON, 
							   32, 32, LR_LOADFROMFILE);
	hIconSm = (HICON) LoadImage (NULL, "sie.ico", IMAGE_ICON, 
							   16, 16, LR_LOADFROMFILE);
	hCursor = (HCURSOR) LoadImage (NULL, IDC_ARROW, IMAGE_CURSOR, 
								   0, 0, LR_SHARED);

// Initializeaza structura WNDCLASSEX
	Wnd.cbSize			= sizeof (WNDCLASSEX);	// dimensiunea structurii in octeti
	Wnd.style			= CS_HREDRAW | CS_VREDRAW;	// stilul clasei
	Wnd.lpfnWndProc		= (WNDPROC) WndProc;	// pointer la functia fereastra
	Wnd.cbClsExtra		= 0;
	Wnd.cbWndExtra		= 0;
	Wnd.hInstance		= hInstance;			// indicator la instanta aplicatiei
	Wnd.hIcon			= hIcon;
	Wnd.hCursor			= hCursor;
	Wnd.hbrBackground	= (HBRUSH) (COLOR_BTNFACE+1);
	Wnd.lpszMenuName	= MAKEINTRESOURCE (IDR_SIEMENU1);
												// pointer la numele meniului
	Wnd.lpszClassName	= szClassName;			// pointer la numele clasei
	Wnd.hIconSm			= hIconSm;

// Inregistreaza clasa de ferestre
	if (!RegisterClassEx (&Wnd)) {
		MessageBox (NULL, "Eroare la inregistrarea clasei",
					"Eroare!", MB_OK | MB_ICONERROR);
		return 0;
	}

// Creeaza fereastra aplicatiei
	hWnd = CreateWindowEx (
		WS_EX_CLIENTEDGE,						// stilul extins
		szClassName,							// pointer la numele clasei
		"Aplicatie SIE",						// titlul ferestrei
		WS_OVERLAPPEDWINDOW,					// stilul ferestrei
		GetSystemMetrics (SM_CXSCREEN)/4,		// pozitia pe orizontala
		GetSystemMetrics (SM_CYSCREEN)/4,		// pozitia pe verticala
		GetSystemMetrics (SM_CXSCREEN)/2,		// latimea ferestrei
		GetSystemMetrics (SM_CYSCREEN)/2,		// inaltimea ferestrei
		NULL,									// indicator la fereastra parinte
		NULL,									// indicator la meniul ferestrei
		hInstance,								// indicator la instanta aplicatiei
		NULL);									// pointer la o valoare transmisa
												// ferestrei
	if (hWnd == NULL) {
		MessageBox (NULL, "Eroare la crearea ferestrei aplicatiei",
					"Eroare", MB_OK | MB_ICONERROR);
		return 0;
	}

// Afiseaza fereastra
	ShowWindow (hWnd, nCmdShow); 
	UpdateWindow (hWnd);

// Bucla de mesaje
	while (GetMessage (&Msg, NULL, 0, 0) > 0) {
		TranslateMessage (&Msg);
		DispatchMessage (&Msg);
	}
	return Msg.wParam;
}

//--------------------------------------------------------------------
// Functia de dialog About
//--------------------------------------------------------------------

BOOL CALLBACK AboutDlgProc (HWND hWnd, UINT uMes,
						    WPARAM wParam, LPARAM lParam)
{
	switch (uMes) {
	case WM_INITDIALOG:
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD (wParam)) {
		case IDOK:
			EndDialog (hWnd, IDOK);
			return TRUE;
		case IDCANCEL:
			EndDialog (hWnd, IDCANCEL);
			return TRUE;
		}
		break;
	}
	return FALSE;
}

//--------------------------------------------------------------------
// Functia fereastra
//--------------------------------------------------------------------

LRESULT CALLBACK WndProc (HWND hWnd, UINT uMes, WPARAM wParam, LPARAM lParam)
{
	HDC hDc;
	HWND hDisplay;
	switch (uMes) {
	case WM_COMMAND:
		switch (LOWORD (wParam)) {
		case ID_FILE_EXIT:
			PostMessage (hWnd, WM_CLOSE, 0, 0);
			break;
		case ID_FILE_OPEN:
			ExecFileOpen (hWnd);
			break;
		case ID_FILE_CLOSE:
			hDisplay = GetDlgItem (hWnd, IDC_DISPLAY);
			DestroyWindow (hDisplay);
			break;
		case ID_HELP_ABOUT:
			DialogBox (GetModuleHandle (NULL),
				MAKEINTRESOURCE (IDD_ABOUT), hWnd, AboutDlgProc);
			break;
		case ID_OPER_OP0:
			hDc = InitTextOut (hWnd);
//--------------------------------------------------------------------
			TestPort (hWnd, hDc);
//--------------------------------------------------------------------
			ReleaseDC (hWnd, hDc);
			break;
		case ID_OPER_OP1:
			hDc = InitTextOut (hWnd);
//--------------------------------------------------------------------
//			De completat: Apelul functiei 1
			MessageBox (hWnd, "Nici o operatie de executat!",
						"Aplicatie SIE", MB_OK | MB_ICONINFORMATION);
//--------------------------------------------------------------------
			ReleaseDC (hWnd, hDc);
			break;
		case ID_OPER_OP2:
			hDc = InitTextOut (hWnd);
//--------------------------------------------------------------------
//			De completat: Apelul functiei 2
			MessageBox (hWnd, "Nici o operatie de executat!",
						"Aplicatie SIE", MB_OK | MB_ICONINFORMATION);
//--------------------------------------------------------------------
			ReleaseDC (hWnd, hDc);
			break;
		case ID_OPER_OP3:
			hDc = InitTextOut (hWnd);
//--------------------------------------------------------------------
//			De completat: Apelul functiei 3
			MessageBox (hWnd, "Nici o operatie de executat!",
						"Aplicatie SIE", MB_OK | MB_ICONINFORMATION);
//--------------------------------------------------------------------
			ReleaseDC (hWnd, hDc);
			break;
		}
		return 0;
	case WM_CLOSE:
		DestroyWindow (hWnd);
		return 0;
	case WM_DESTROY:
		PostQuitMessage (0);
		return 0;
	default:
		return DefWindowProc (hWnd, uMes, wParam, lParam);
	}
}
